/*
 ************************************************************************************************
 *                                                                                              *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 *
 *                                                                                              *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro                              *
 *                  http://picosa.narod.ru                                                      *
 *                                                                                              *
 *----------------------------------------------------------------------------------------------*
 *                                                                                              *
 *  File:           osa_ccs18.h                                                                 *
 *                                                                                              *
 *  Compilers:      CCS                                                                         *
 *                                                                                              *
 *  Programmer:     Timofeev Victor                                                             *
 *                  osa@pic24.ru, testerplus@mail.ru                                            *
 *                                                                                              *
 *  Description:    CCS18 specific definitions                                                  *
 *                                                                                              *
 *  History:        21.01.2009                                                                  *
 *                  01.05.2009 -    All processor specific definitions moved from osa.h into    *
 *                                  this file                                                   *
 *                                                                                              *
 ************************************************************************************************
 */


#ifndef __OSACCS18_H__
#define __OSACCS18_H__




/************************************************************************************************
 *                                                                                              *
 *     Integer types definitions                                                                * 
 *                                                                                              *
 ************************************************************************************************/

#define __osa_uint8     unsigned int8
#define __osa_uint16    unsigned int16
#define __osa_uint32    unsigned int32

#define _OST_INT_TYPE   __osa_uint8
#define _OST_INT_MASK    7
#define _OST_INT_SHIFT   3
#define _OST_INT_SIZE    8



/************************************************************************************************
 *                                                                                              * 
 *     Location keywords                                                                        * 
 *                                                                                              * 
 ************************************************************************************************/

#define NEAR
#define bank0
#define bank1
#define bank2
#define bank3

#define bit             _OST_INT_TYPE
#define __osa30_near

#define _OS_CUR_FLAGS_IN_OS_state   // Work with tasks state througth OS_state variable



/************************************************************************************************
 *                                                                                              * 
 *     Processor specified definitions for system flags                                         * 
 *     (see OST_SYSTEM_FLAGS and OST_TCB definition in file "OSA.h")                            * 
 *                                                                                              * 
 ************************************************************************************************/

#define _OST_SYSTEM_FLAGS_PROC_SPEC                                                             \
                                                                                                \
        /*5*/   _OST_INT_TYPE   bTimeout    : 1; /* Event waiting timeout                */     \
        /*6*/   _OST_INT_TYPE   bGIE_CTemp  : 1; /* Saved GIEH while in critical section */     \
        /*7*/   _OST_INT_TYPE   bGIEL_CTemp : 1; /* Saved GIEL while in critical section */     \

#define _OST_TCB_PROC_SPEC      // Empty macro for this processor


/************************************************************************************************
 *                                                                                              *
 *     Registers definitions                                                                    *
 *                                                                                              *
 ************************************************************************************************/

#word _fsr      = 0xFE9
#byte _fsrl     = 0xFE9
#byte _indf     = 0xFEF
#byte _postinc  = 0xFEE
#byte _postdec  = 0xFED
#byte _preinc   = 0xFEC 

#byte _fsr1l    = 0xFE1

#byte _pcl      = 0xFF9
#byte _pclath   = 0xFFA
#byte _pclatu   = 0xFFB
#byte _status   = 0xFD8
#byte _tosl     = 0xFFD
#byte _tosh     = 0xFFE
#byte _tosu     = 0xFFF
#byte _bsr      = 0xFE0
#byte _wreg     = 0xFE8
#byte _intcon   = 0xFF2
#byte _rcon     = 0xFD0
#byte _stkptr   = 0xFFC


#bit  _gie      = _intcon.7
#bit  _gieh     = _intcon.7
#bit  _giel     = _intcon.6
#bit  _ipen     = _rcon.7


/************************************************************************************************
 *                                                                                              *
 *     Constants and types                                                                      *
 *                                                                                              *
 ************************************************************************************************/

#ifndef OS_SMSG_SIZE
#define OS_SMSG_SIZE    0
#endif

//------------------------------------------------------------------------------

#define _FSR_TYPE            __osa_uint16

//------------------------------------------------------------------------------

#define __SMALL__

#if defined(__SMALL__)
    #define OST_CODE_POINTER    __osa_uint16
    #define OS_CODE_POINTER_SIZE    2
#else
    #define OST_CODE_POINTER    __osa_uint32
    #define OS_CODE_POINTER_SIZE    4
#endif




/************************************************************************************************
 *                                                                                              *
 *     Platform specific macros                                                                 * 
 *                                                                                              *
 ************************************************************************************************/


#define _PIC18_ERRATA_NOP()  #asm nop #endasm
#define _PIC18_POP()         #asm pop #endasm

#define OS_CLRWDT()   #asm  clrwdt  #endasm
#define OS_SLEEP()    #asm  sleep   #endasm









/************************************************************************************************
 *                                                                                              * 
 *     Internal macro for work with state flags                                                 * 
 *                                                                                              * 
 ************************************************************************************************/

//#define _OS_bTaskReady              ((OST_TASK_STATE)_indf).bReady
//#define _OS_bTaskCanContinue        ((OST_TASK_STATE)_indf).bCanContinue
//#define _OS_bTaskDelay              ((OST_TASK_STATE)_indf).bDelay
//#define _OS_bTaskTimeout            (!(((OST_TASK_STATE)_indf).bDelay))
//#define _OS_bTaskEnable             ((OST_TASK_STATE)_indf).bEnable
//#define _OS_cTaskPriority           ((OST_TASK_STATE)_indf).cPriority
//#define _OS_bTaskPaused             ((OST_TASK_STATE)_indf).bPaused



/************************************************************************************************
 *                                                                                              *
 *     Context switching macros                                                                 *
 *                                                                                              *
 ************************************************************************************************/

//------------------------------------------------------------------------------
// Save/restore PC macros
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#if OS_CODE_POINTER_SIZE == 2
//------------------------------------------------------------------------------

    #define _OS_SetPC()                     \
        #asm                                \
            movff      _postinc, _wreg      \
            movff      _postinc, _pclath    \
            movwf     _pcl                  \
        #endasm

    
    #define _OS_SavePC()                    \
        _postdec = _tosh;                   \
        _postdec = _tosl;

//------------------------------------------------------------------------------
#else
//------------------------------------------------------------------------------

    #define _OS_SetPC()                     \
        #asm                                \
            movff      _postinc, _wreg      \
            movff      _postinc, _pclath    \
            movff      _postinc, _pclatu    \
            movwf     _pcl                  \
        #endasm

    #define _OS_SavePC()                    \
        _postdec = 0;                       \
        _postdec = _tosu;                   \
        _postdec = _tosh;                   \
        _postdec = _tosl;

//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------

#separate
void __OS_JumpToTask (void);


#define _OS_JumpToTask()                                                    \
    {                                                                       \
        __OS_JumpToTask();                                                  \
    }

#define _OS_SaveRetAddr()                                                   \
        _OS_SET_FSR_CUR_TASK();                                             \
        _indf &= 0xD7;                                                      \
        _indf |= ((char)OS_state) & 0x28;                                   \

    
#define _OS_if_Ready()  _OS_SET_FSR_CUR_TASK();                             \
                        if (_OS_CheckIndfReady())   


 
//------------------------------------------------------------------------------

extern #separate void OS_SchedRetPoint (void);
extern #separate void OS_SchedRetPointNoSave (void);
extern #separate void OS_ClearReady (void);
extern #separate void OS_ClearReadySetCanContinue (void);

    
//------------------------------------------------------------------------------
#define _OS_RETURN_SAVE()                       \
    {                                           \
        OS_SchedRetPoint();                     \
        _PIC18_ERRATA_NOP();                    \
    }

//------------------------------------------------------------------------------
#define _OS_CLEAR_READY()                       \
    {                                           \
        OS_ClearReady();                        \
        _PIC18_ERRATA_NOP();                    \
    }
//------------------------------------------------------------------------------
#define _OS_CLEAR_READY_SET_CANCONTINUE()       \
    {                                           \
        OS_ClearReadySetCanContinue();          \
        _PIC18_ERRATA_NOP();                    \
    }
//------------------------------------------------------------------------------
#define _OS_RETURN_NO_SAVE()                    \
    {                                           \
        OS_SchedRetPointNoSave();               \
        _PIC18_ERRATA_NOP();                    \
    }                                           \
//------------------------------------------------------------------------------


#define _OS_SET_FSR_CUR_TASK()      _fsr = (_FSR_TYPE)&_OS_CurTask->State
#define _OS_SET_FSR_CUR_TASK_W(w)   _fsr = ((_FSR_TYPE)&_OS_CurTask->State) + w
 



















//______________________________________________________________________________
/******************************************************************************************
 *
 *                                                
 *
 ******************************************************************************************/


 
 
/******************************************************************************************
 *
 *  WORK WITH DYNAMIC TIMERS
 *
 ******************************************************************************************/

//------------------------------------------------------------------------------
#ifdef OS_ENABLE_DTIMERS
//------------------------------------------------------------------------------

#define _OS_DTimersWork_DEFINED

#define _OS_IndfTimerTimeout       ((NEAR OST_DTIMER_FLAGS)(_indf)).bTimeout
#define _OS_IndfTimerActive        ((NEAR OST_DTIMER_FLAGS)(_indf)).bActive
#define _OS_IndfTimerRun           ((NEAR OST_DTIMER_FLAGS)(_indf)).bRun
#define _OS_IndfTimerNextEnable    ((NEAR OST_DTIMER_FLAGS)(_indf)).bNextEnable
#define _OS_PostincTimerNextEnable ((NEAR OST_DTIMER_FLAGS)(_postinc)).bNextEnable





#if   OS_TIMER_SIZE == 1
    #define OS_INC_DTIMER()                                                             \
        {                                                                               \
            #asm                                                                        \
                movlw   3                                                               \
                addwf   _fsr, f                                                         \
                incf    _postdec, f     /*      FSR0L */    \
                movf    _postdec, w                                                     \
                movf    _postdec, w                                                     \
            #endasm                                                                     \
            if (_status & 0x01) _OS_IndfTimerTimeout = 1;                               \
        }

        
#elif OS_TIMER_SIZE == 2
    #define OS_INC_DTIMER()                                                             \
        {                                                                               \
            _OS_DTimers.Flags.bTimeout = 0;                                             \
            #asm                                                                        \
                movlw   3                                                               \
                addwf   _fsr, f                                                         \
                movlw   0                                                               \
                incf    _postinc, f                                                     \
                addwfc  _postdec, f                                                     \
            #endasm                                                                     \
            if (_status & 0x01) _OS_DTimers.Flags.bTimeout = 1;                         \
            #asm                                                                        \
                movlw   3                                                               \
                subwf   _fsr, f                                                         \
            #endasm                                                                     \
            if (_OS_DTimers.Flags.bTimeout)                                             \
                _OS_IndfTimerTimeout = 1;                                               \
        }


#elif OS_TIMER_SIZE == 4
    #define OS_INC_DTIMER()                                                             \
        {                                                                               \
            _OS_DTimers.Flags.bTimeout = 0;                                             \
            #asm                                                                        \
                movlw   3                                                               \
                addwf   _fsr, f                                                         \
                movlw   0                                                               \
                incf    _postinc, f                                                     \
                addwfc  _postinc, f                                                     \
                addwfc  _postinc, f                                                     \
                addwfc  _postdec, f                                                     \
            #endasm                                                                     \
            if (_status & 0x01) _OS_DTimers.Flags.bTimeout = 1;                         \
            #asm                                                                        \
                movlw   5                                                               \
                subwf   _fsr, f                                                         \
            #endasm                                                                     \
            if (_OS_DTimers.Flags.bTimeout)                                             \
                _OS_IndfTimerTimeout = 1;                                               \
        }

#endif


#define __OS_DTimersWork()                              \
    {                                                   \
        _fsr = (_FSR_TYPE) &_OS_DTimers.Flags;          \
    REPEAT:                                             \
        if (_OS_PostincTimerNextEnable)                 \
        {                                               \
            /*  FSR  .  */     \
            #asm                                        \
                movf    _postinc, w                     \
                movff   _postdec, &_fsr + 1             \
                movwf   &_fsr + 0                       \
            #endasm                                     \
            if (!_OS_IndfTimerRun) goto REPEAT;         \
            OS_INC_DTIMER();                            \
            goto REPEAT;                                \
        }                                               \
    }


//------------------------------------------------------------------------------
#endif  // OS_ENABLE_DTIMER
//------------------------------------------------------------------------------





/************************************************************************************************
 *                                                                                              * 
 *     Create, replace, reserve task specific services                                          * 
 *                                                                                              * 
 ************************************************************************************************/


/************************************************************************/
/*                                                                      */
/* Replace current task with new (current task will be deleted)         */
/*                                                                      */
/************************************************************************/

#define OS_Task_Replace(priority, TaskName)  qewrqwerqwer


/************************************************************************/
/*                                                                      */
/* Create task                                                          */
/*                                                                      */
/************************************************************************/

#define OS_Task_Create(priority, TaskName)                          \
    {                                                               \
        _OS_Task_Create(priority, (OST_CODE_POINTER)TaskName);      \
        if (_pcl & 1) TaskName();                                   \
    }


/************************************************************************/
/*                                                                      */
/* Create task by indirrect address                                     */
/*                                                                      */
/************************************************************************/

#define OS_Task_Reserve(TaskName)





/************************************************************************************************
 *                                                                                              *
 *                                     I N T E R R U P T S                                      * 
 *                                                                                              *
 ************************************************************************************************/

extern char OS_DI (void);
extern void OS_RI (char);
#define OS_EI()         { _gie = 1; if (_ipen) _giel = 1;}
#define OS_EIH()        { _gie = 1; }
#define OS_EIL()        { _giel = 1; }




/************************************************************************************************
 *                                                                                              *
 *                                     C O N D I T I O N S                                      *
 *                                                                                              *
 ************************************************************************************************/


extern #separate void _OS_CheckEvent (_OST_INT_TYPE);

#define _OS_CHECK_EVENT(event) _OS_CheckEvent(event)







#endif
